unit AMsgBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TAMBehavior = (amShowUnchecked, amShowChecked);
  TAdvMsgBox = class(TComponent)
  private
    { Private-Deklarationen }
  protected
    { Protected-Deklarationen }
    fText: TStrings;
    fTitle, fCBCaption, fRegKey: String;
    fCBCheck: Boolean;
    fRegResult: Integer;
    fBehave: TAMBehavior;
    fButtons: TMsgDlgButtons;

    procedure SetText(val: TStrings);
    function  GetRegCheck: Boolean;
    function  GetRegResult: Integer;
    procedure SetRegKey(val: String);
    function  HaveRegEntries: Boolean;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    function  Execute: Integer;
    procedure DeleteRegEntries;
  published
    { Published-Deklarationen }
    property Text: TStrings read fText write SetText;
    property Title: String read fTitle write fTitle;
    property CBoxCaption: String read fCBCaption write fCBCaption;
    property CBoxChecked: Boolean read fCBCheck write fCBCheck;
    property RegKey: String read fRegKey write SetRegKey;
    property Result: Integer read GetRegResult;
    property Behavior: TAMBehavior read fBehave write fBehave;
    property Buttons: TMsgDlgButtons read fButtons write fButtons;
  end;

procedure Register;

implementation
uses StdCtrls, registry;

const REG_CHECK: String = 'Check';
      REG_RESULT: String = 'Result';

procedure Register;
begin
  RegisterComponents('XWatch', [TAdvMsgBox]);
end;

constructor TAdvMsgBox.Create(AOwner: TComponent);
begin
     inherited;
     fTitle:='XWatch - The truth is not out there!';
     fCBCaption:='Do not show this message again.';
     fCBCheck:=GetRegCheck;
     fText:=TStringList.Create;
     fRegKey:='';
     fBehave:=amShowUnchecked;
end;

function TAdvMsgBox.Execute: Integer;
var dlg: TForm;
    cb: TCheckBox;
    mytop,i: Integer;
    lab:TLabel;
    reg: TRegistry;
begin
     if HaveRegEntries and
        (((FBehave=amShowUnchecked) and fCBCheck) or
        ((FBehave=amShowChecked) and not fCBCheck)) then
     begin
          result:=fRegResult;
          exit;
     end;
     dlg:=CreateMessageDialog(fText.Text, mtWarning, fButtons);
     cb:=TCheckBox.Create(dlg);
     lab:=TLabel.Create(dlg);

     lab.Parent:=dlg;
     lab.Visible:=false;
     lab.Caption:=fCBCaption;

     cb.Caption:=fCBCaption;
     cb.Parent:=dlg;
     cb.Checked:=fCBCheck;
     cb.Width:=lab.Width+30;

     mytop:=0;
     // Position checkbox above buttons.
     for i:=0 to dlg.ControlCount-1 do
     begin
          if dlg.Controls[i] is TButton then
          begin
               mytop:=dlg.Controls[i].Top;
               // move buttons down
               dlg.Controls[i].Top:=dlg.Controls[i].Top+cb.Height;
          end;
     end;

     cb.Top:=mytop-10;
     dlg.Height:=dlg.Height+cb.Height;

     if dlg.Width<cb.Width then
     begin
          {TODO: Wenn man die Breite ndert,
           sind die Buttons nicht mehr richtig positioniert.}

          //dlg.Width:=cb.Width+50;
     end else cb.Left:=dlg.Width div 2-cb.Width div 2;

     dlg.Caption:=fTitle;
     fRegResult:=dlg.ShowModal;
     fCBCheck:=cb.Checked;

     if fRegKey='' then exit;
     reg := TRegistry.Create;
     try
        Reg.RootKey := HKEY_CURRENT_USER;
        if Reg.OpenKey(fRegKey, True) then
        begin
           reg.WriteBool(name+REG_CHECK,fCBCheck);
           reg.WriteInteger(name+REG_RESULT,fRegResult);
        end;
     finally
        Reg.CloseKey;
        Reg.Free;
     end;
     result:=fRegResult;
end;

procedure TAdvMsgBox.SetText(val: TStrings);
begin
     fText.Assign(val);
end;


function TAdvMsgBox.HaveRegEntries: Boolean;
var reg: TRegistry;
begin
     result:=false;
     if fRegKey<>'' then
     begin
          reg:=TRegistry.Create;
          try
             Reg.RootKey := HKEY_CURRENT_USER;
             if reg.OpenKey(fRegKey, false) then
             begin
                  result:=reg.ValueExists(Name+REG_CHECK) and
                          reg.ValueExists(Name+REG_RESULT);
             end ;
          finally
             Reg.CloseKey;
             reg.Free;
          end;
     end;
end;

function TAdvMsgBox.GetRegCheck: Boolean;
var reg: TRegistry;
begin
    if not HaveRegEntries then exit; // TODO: Exception
    reg:=TRegistry.Create;
    result:=false;
    try
       Reg.RootKey := HKEY_CURRENT_USER;
       if reg.OpenKey(fRegKey, false) then
            result:=reg.ReadBool(Name+REG_CHECK);
    finally
       Reg.CloseKey;
       reg.Free;
    end;
end;

function TAdvMsgBox.GetRegResult: Integer;
var reg: TRegistry;
begin
    if not HaveRegEntries then exit; // No result? Hmmm...
    reg:=TRegistry.Create;
    result:=0;
    try
       Reg.RootKey := HKEY_CURRENT_USER;
       if reg.OpenKey(fRegKey, false) then
            result:=reg.ReadInteger(Name+REG_RESULT);
    finally
       Reg.CloseKey;
       reg.Free;
    end;
end;

procedure TAdvMsgBox.SetRegKey(val: String);
begin
     if(val<>fRegKey) then
     begin
          fRegKey:=val;
          if HaveRegEntries then
          begin
               fRegResult:=GetRegResult;
               fCBCheck:=GetRegCheck;
          end;
     end;
end;

procedure TAdvMsgBox.DeleteRegEntries;
var reg: TRegistry;
begin
     if not HaveRegEntries then exit;
     if fRegKey<>'' then
     begin
          reg:=TRegistry.Create;
          try
             Reg.RootKey := HKEY_CURRENT_USER;
             if reg.OpenKey(fRegKey, false) then
             begin
                  reg.DeleteValue(Name+REG_CHECK);
                  reg.DeleteValue(Name+REG_RESULT);
             end ;
          finally
             Reg.CloseKey;
             reg.Free;
          end;
     end;
     // Settings back to default
     fCBCheck:=fBehave<>amShowUnchecked;
     fRegResult:=0;
end;
end.
